# Plugin for Foswiki - The Free and Open Source Wiki, http://foswiki.org/
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at
# http://www.gnu.org/copyleft/gpl.html

=pod

---+ package Foswiki::Plugins::FwTopicPermissionsPlugin

When developing a plugin it is important to remember that

Foswiki is tolerant of plugins that do not compile. In this case,
the failure will be silent but the plugin will not be available.
See %SYSTEMWEB%.InstalledPlugins for error messages.

__NOTE:__ Foswiki:Development.StepByStepRenderingOrder helps you decide which
rendering handler to use. When writing handlers, keep in mind that these may
be invoked

on included topics. For example, if a plugin generates links to the current
topic, these need to be generated before the =afterCommonTagsHandler= is run.
After that point in the rendering loop we have lost the information that
the text had been included from another topic.

=cut


package Foswiki::Plugins::FwTopicPermissionsPlugin;

# Always use strict to enforce variable scoping
use strict;

require Foswiki::Func;    # The plugins API
require Foswiki::Plugins; # For the API version

# $VERSION is referred to by Foswiki, and is the only global variable that
# *must* exist in this package.
# This should always be $Rev: 3048 (2009-03-12) $ so that Foswiki can determine the checked-in
# status of the plugin. It is used by the build automation tools, so
# you should leave it alone.
our $VERSION = '$Rev: 3048 (2009-03-12) $';

# This is a free-form string you can use to "name" your own plugin version.
# It is *not* used by the build automation tools, but is reported as part
# of the version number in PLUGINDESCRIPTIONS.
our $RELEASE = '$Date: 2008-12-14 18:49:56 +0100 (Sun, 14 Dec 2008) $';

# Short description of this plugin
# One line description, is shown in the %SYSTEMWEB%.TextFormattingRules topic:
our $SHORTDESCRIPTION = 'lets users easily set permissions for single topics';

# You must set $NO_PREFS_IN_TOPIC to 0 if you want your plugin to use
# preferences set in the plugin topic. This is required for compatibility
# with older plugins, but imposes a significant performance penalty, and
# is not recommended. Instead, leave $NO_PREFS_IN_TOPIC at 1 and use
# =$Foswiki::cfg= entries set in =LocalSite.cfg=, or if you want the users
# to be able to change settings, then use standard Foswiki preferences that
# can be defined in your %USERSWEB%.SitePreferences and overridden at the web
# and topic level.
our $NO_PREFS_IN_TOPIC = 0;

=begin TML

---++ initPlugin($topic, $web, $user) -> $boolean
   * =$topic= - the name of the topic in the current CGI query
   * =$web= - the name of the web in the current CGI query
   * =$user= - the login name of the user
   * =$installWeb= - the name of the web the plugin topic is in
     (usually the same as =$Foswiki::cfg{SystemWebName}=)

*REQUIRED*

Called to initialise the plugin. If everything is OK, should return
a non-zero value. On non-fatal failure, should write a message
using =Foswiki::Func::writeWarning= and return 0. In this case
%<nop>FAILEDPLUGINS% will indicate which plugins failed.

In the case of a catastrophic failure that will prevent the whole
installation from working safely, this handler may use 'die', which
will be trapped and reported in the browser.

__Note:__ Please align macro names with the Plugin name, e.g. if
your Plugin is called !FooBarPlugin, name macros FOOBAR and/or
FOOBARSOMETHING. This avoids namespace issues.

=cut

sub initPlugin {
	my( $topic, $web, $user, $installWeb ) = @_;

	# check for Plugins.pm versions
	if( $Foswiki::Plugins::VERSION < 2.0 ) {
		Foswiki::Func::writeWarning( 'Version mismatch between ',
					__PACKAGE__, ' and Plugins.pm' );
		return 0;
	}

	my( $topic, $web, $user, $installWeb ) = @_;
	my $binPath = Foswiki::Func::getScriptUrl();
	my $exclusiveMode = Foswiki::Func::getPluginPreferencesValue('EXCLUSIVEMODE');
	my $saveButtonMode = Foswiki::Func::getPluginPreferencesValue('SAVEBUTTONMODE');
	my $exclusiveModeString = '';
	my $saveButtonModeString = '';

	if (lc($exclusiveMode) eq 'true' || $exclusiveMode eq '1' || lc($exclusiveMode) eq 'yes') {
		$exclusiveModeString = 'true';
	} else {
		$exclusiveModeString = 'false';
	}

	if (lc($saveButtonMode) eq 'true' || $saveButtonMode eq '1' || lc($saveButtonMode) eq 'yes') {
		$saveButtonModeString = 'true';
	} else {
		$saveButtonModeString = 'false';
	}

	# Display the link only if the current user is in AdminGroup
	if (Foswiki::Func::isAnAdmin($user)) {
		Foswiki::Func::addToHEAD('TOPICPERMISSIONS_VARS', '<script type="text/javascript">var topicPermissionsBinPath = "'.$binPath.'"; var topicPermissionsWeb = "'.$web.'"; var topicPermissionsTopic = "'.$topic.'"; var exclusiveMode = '.$exclusiveModeString.'; var saveButtonMode = '.$saveButtonModeString.';</script>');
		Foswiki::Func::addToHEAD('TOPICPERMISSIONS_SCRIPT', '<script type="text/javascript" src="%PUBURLPATH%/%SYSTEMWEB%/FwTopicPermissionsPlugin/FwTopicPermissions.js"></script>');
		Foswiki::Func::addToHEAD('TOPICPERMISSIONS_CSS', '<link rel="stylesheet" type="text/css" href="%PUBURLPATH%/%SYSTEMWEB%/FwTopicPermissionsPlugin/style.css"></script>');

		Foswiki::Func::registerRESTHandler('getUserList', \&get_user_list);
		Foswiki::Func::registerRESTHandler('savePrefs', \&save_prefs);
	}

	# Plugin correctly initialized
	return 1;
}

sub get_user_list {
	my $query = Foswiki::Func::getCgiQuery();
	my $web = $query->param('web');
	my $topic = $query->param('topic');
	my $mode = $query->param('mode');
	my $output = '<table id="topicpermissions_'.$mode.'table">';

	my ($meta, $text) = Foswiki::Func::readTopic($web, $topic);

	# get ALLOW preferences
	my $allow_topic_rename_ref = $meta->get( 'PREFERENCE', 'ALLOWTOPICRENAME' );
	my @allow_topic_rename = split(/,\ */, $allow_topic_rename_ref->{'value'});

	my $allow_topic_view_ref = $meta->get( 'PREFERENCE', 'ALLOWTOPICVIEW' );
	my @allow_topic_view = split(/,\ */, $allow_topic_view_ref->{'value'});

	my $allow_topic_change_ref = $meta->get( 'PREFERENCE', 'ALLOWTOPICCHANGE' );
	my @allow_topic_change = split(/,\ */, $allow_topic_change_ref->{'value'});

	# get DENY preferences
	my $deny_topic_rename_ref = $meta->get( 'PREFERENCE', 'DENYTOPICRENAME' );
	my @deny_topic_rename = split(/,\ */, $deny_topic_rename_ref->{'value'});

	my $deny_topic_view_ref = $meta->get( 'PREFERENCE', 'DENYTOPICVIEW' );
	my @deny_topic_view = split(/,\ */, $deny_topic_view_ref->{'value'});

	my $deny_topic_change_ref = $meta->get( 'PREFERENCE', 'DENYTOPICCHANGE' );
	my @deny_topic_change = split(/,\ */, $deny_topic_change_ref->{'value'});

	my @userArr = ();
        my $it;

	if ($mode eq "user") {
		$it = Foswiki::Func::eachUser();
	} else {
		$it = Foswiki::Func::eachGroup();
	}

	while ($it->hasNext()) {
		my $user = $it->next();
		# we don't want those users listed
		if ($user ne 'ProjectContributor' && $user ne 'RegistrationAgent' && $user ne 'AdminUser') {
			push(@userArr, $user);
		}
	}

	my @userArr_sort = sort @userArr;

	foreach my $user (@userArr_sort) {
		my $allow_rename = "";
		my $allow_view = "";
		my $allow_change = "";

		my $deny_rename = "";
		my $deny_view = "";
		my $deny_change = "";

		# check if user is in @allow_topic_rename
		if(grep $_ eq $user, @allow_topic_rename) {
			$allow_rename = "tp_rename_allow";
		}

		if(grep $_ eq $user, @allow_topic_view) {
			$allow_view = "tp_view_allow";
		}

		if(grep $_ eq $user, @allow_topic_change) {
			$allow_change = "tp_change_allow";
		}

		if(grep $_ eq $user, @deny_topic_rename) {
			$deny_rename = "tp_rename_deny";
		}

		if(grep $_ eq $user, @deny_topic_view) {
			$deny_view = "tp_view_deny";
		}

		if(grep $_ eq $user, @deny_topic_change) {
			$deny_change = "tp_change_deny";
		}

		my $id_classes = $allow_rename.' '.$allow_view.' '.$allow_change.' '.$deny_rename.' '.$deny_view.' '.$deny_change;

		$output .= '<tr id="'.$user.'"><td><input id="'.$user.'_check" type="checkbox" class="'.$id_classes.'"/></td><td>'.$user.'</td></tr>';
	}
	return $output;
}


sub save_prefs {
	my $query = Foswiki::Func::getCgiQuery();
	my $web = Unicode::MapUTF8::from_utf8({ -string => $query->param('web'), -charset => 'ISO-8859-1' });
	my $topic = Unicode::MapUTF8::from_utf8({ -string => $query->param('topic'), -charset => 'ISO-8859-1' });
	my $realtopicname = Unicode::MapUTF8::from_utf8({ -string => $query->param('realtopicname'), -charset => 'ISO-8859-1' });

	my $view_allow = $query->param('view_allow');
	my $change_allow = $query->param('change_allow');
	my $rename_allow = $query->param('rename_allow');

	my $view_deny = $query->param('view_deny');
	my $change_deny = $query->param('change_deny');
	my $rename_deny = $query->param('rename_deny');

	my ($meta, $text) = Foswiki::Func::readTopic($web, $topic);


	$meta->putKeyed( 'PREFERENCE', { name => 'ALLOWTOPICVIEW', title => 'ALLOWTOPICVIEW', value => $view_allow } );
	$meta->putKeyed( 'PREFERENCE', { name => 'ALLOWTOPICCHANGE', title => 'ALLOWTOPICCHANGE', value => $change_allow } );
	$meta->putKeyed( 'PREFERENCE', { name => 'ALLOWTOPICRENAME', title => 'ALLOWTOPICRENAME', value => $rename_allow } );

	$meta->putKeyed( 'PREFERENCE', { name => 'DENYTOPICVIEW', title => 'DENYTOPICVIEW', value => $view_deny } );
	$meta->putKeyed( 'PREFERENCE', { name => 'DENYTOPICCHANGE', title => 'DENYTOPICCHANGE', value => $change_deny } );
	$meta->putKeyed( 'PREFERENCE', { name => 'DENYTOPICRENAME', title => 'DENYTOPICRENAME', value => $rename_deny } );

	Foswiki::Func::saveTopic($web, $realtopicname, $meta, $text);
}

1;
__END__
# This copyright information applies to the FwTopicPermissionsPlugin:

# Plugin for Foswiki - The Free and Open Source Wiki, http://foswiki.org/
#
# FwTopicPermissionsPlugin is # This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# For licensing info read LICENSE file in the Foswiki root.
